package com.ieng.rms.cntroller;


import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.ieng.rms.dto.AlarmDetailReportDto;
import com.ieng.rms.entity.AlarmDetailConfig;
import com.ieng.rms.entity.AlarmDetails;
import com.ieng.rms.models.SiteDetailsDTO;
import com.ieng.rms.services.AlarmDetailConfigService;
import com.ieng.rms.services.CommonService;
import com.ieng.rms.services.RmsCoreClientService;
import com.ieng.rms.services.SiteWiseAlarmMaskingConfigService;
import com.ieng.rms.util.URL;
import com.ieng.rms.util.Util;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.codec.ServerSentEvent;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;

import java.time.Duration;
import java.util.*;
import java.util.stream.Collectors;

import static com.ieng.rms.util.Constants.ALARM_CONFIG_MAP_FOR_NOC;

@CrossOrigin("*")
@RestController
@RequestMapping(URL.DEFAULT + URL.NOC_ALARMS)
public class NocAlarmMonitorController {

    @Autowired
    CommonService commonService;

    @Autowired
    AlarmDetailConfigService alarmDetailConfigService;

    @Autowired
    RmsCoreClientService rmsCoreClientService;

    private final SiteWiseAlarmMaskingConfigService siteWiseAlarmMaskingConfigService;

    public NocAlarmMonitorController(SiteWiseAlarmMaskingConfigService siteWiseAlarmMaskingConfigService) {
        this.siteWiseAlarmMaskingConfigService = siteWiseAlarmMaskingConfigService;
    }


    @GetMapping(path = URL.NOC_ALARMS_MONITOR, produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    public Flux<ServerSentEvent<Collection<AlarmDetailReportDto>>> monitor(@RequestParam Integer contractId, @RequestParam(value = "operatorIds", required = false) List<Integer> operatorIds) {
        HttpHeaders headers = new HttpHeaders();

        Integer finalContractId = contractId;

        return Flux.interval(Duration.ofSeconds(5)).map(sec -> sec == 0 ?
                        getContractAlarms(finalContractId, operatorIds) : getContractAlarms2(finalContractId, operatorIds))
                .map(model -> ServerSentEvent.<Collection<AlarmDetailReportDto>>builder()
                        .event("New Packet")
                        .data(model)
                        .build())
                .onBackpressureDrop()
                .log();
    }

    // TODO: 7/19/2023
    // temporary Fix for Alaram Detail map. SITES registered with CamelCase and ALARMS MAP with upper case.
    // Issue with matching so temp fix is done at  Line 68.

    private List<AlarmDetailReportDto> getContractAlarms(Integer finalContractId, List<Integer> operatorId) {
        List<String> siteList = getSiteList(finalContractId);
        List<AlarmDetailReportDto> alarmDetailsReport = new ArrayList<>();

        siteList.forEach(site -> {
            SiteDetailsDTO siteDetailsDTO = rmsCoreClientService.getSiteDetailsBySiteId(site);
            Map<String, Map<String, AlarmDetails>> alarmDetails = alarmDetailConfigService.loadAlarmDetailsFromRedisOrDB(siteDetailsDTO.getSiteId());

            Map<String, AlarmDetails> alarmMap = alarmDetails.get(site) == null ? alarmDetails.get(site.toUpperCase()) : alarmDetails.get(site);
            List<AlarmDetailConfig> alarmDetailConfigList = alarmDetailConfigService.getAlarmDetailConfigList(finalContractId);

            if (alarmMap != null && !alarmMap.values().isEmpty()) {
                alarmMap.values().stream()
                        .forEach(alarm -> {
                            Optional<AlarmDetailConfig> alarmDetailConfig = alarmDetailConfigList.stream().filter(p -> Util.removeUnderscoreWithSpace(p.getAlarmName()).equals(alarm.getAlarmName())).findFirst();
                            alarmDetailsReport.add(new AlarmDetailReportDto(alarm, siteDetailsDTO, alarmDetailConfig));
                        });
            }
        });
        alarmDetailsReport.stream().sorted(Comparator.comparing(AlarmDetailReportDto::getFromTime));
        return alarmDetailsReport;
    }
//    private List<AlarmDetailReportDto> getContractAlarms(Integer finalContractId, List<Integer> operatorId) {
//        List<String> siteList = getSiteList(finalContractId);
//        List<AlarmDetailReportDto> alarmDetails = new ArrayList<>();
//
//        siteList.forEach(site -> {
//            SiteDetailsDTO siteDetailsDTO = rmsCoreClientService.getSiteDetailsBySiteId(site);
//            Map<String, AlarmDetails> alarmMap = Constants.alarmDetailMap.get(site) == null ? Constants.alarmDetailMap.get(site.toUpperCase()) : Constants.alarmDetailMap.get(site);
//            List<AlarmDetailConfig> alarmDetailConfigList = alarmDetailConfigService.getAlarmDetailConfigList(finalContractId);
//
//            // Fetch site-wise alarm masking config
//            Map<String, SiteWiseAlarmMaskingConfig> alarmMaskingConfigMap = siteWiseAlarmMaskingConfigService.getAlarmMaskingConfigMap(site);
//
//            if (alarmMap != null && !alarmMap.values().isEmpty()) {
//                alarmMap.values().stream()
//                        .filter(alarm -> {
//                            SiteWiseAlarmMaskingConfig config = alarmMaskingConfigMap.get(alarm.getAlarmName());
//                            return config == null || config.getAlarmEnabled(); // Include only if not in config or explicitly enabled
//                        })
//                        .forEach(alarm -> {
//                            Optional<AlarmDetailConfig> alarmDetailConfig = alarmDetailConfigList.stream()
//                                    .filter(p -> Util.removeUnderscoreWithSpace(p.getAlarmName()).equals(alarm.getAlarmName()))
//                                    .findFirst();
//                            alarmDetails.add(new AlarmDetailReportDto(alarm, siteDetailsDTO, alarmDetailConfig));
//                        });
//            }
//        });
//
//        alarmDetails.sort(Comparator.comparing(AlarmDetailReportDto::getFromTime));
//        return alarmDetails;
//    }


    private List<AlarmDetailReportDto> getContractAlarms2(Integer finalContractId, List<Integer> operatorId) {
        List<String> siteList = getSiteList(finalContractId);
        Map<String, Object> configData = alarmDetailConfigService.getAlarmConfigData();
        Map<String, AlarmDetailConfig> alarmConfigMap = new HashMap<>();
        ObjectMapper mapper = new ObjectMapper();
        mapper.registerModule(new JavaTimeModule());
        Object rawData = configData.get(ALARM_CONFIG_MAP_FOR_NOC);
        if (rawData instanceof Map) {
            try {
                alarmConfigMap = mapper.convertValue(rawData, new TypeReference<>() {
                });
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        List<AlarmDetailReportDto> resultList = new ArrayList<>();
        Map<String, List<AlarmDetailReportDto>> siteWiseAlarmMap = new HashMap<>();
        Map<String, AlarmDetailConfig> finalAlarmConfigMap = alarmConfigMap;
        siteList.forEach(site -> {
            List<AlarmDetailReportDto> alarmDetails = new ArrayList<>();
            SiteDetailsDTO siteDetailsDTO = rmsCoreClientService.getSiteDetailsBySiteId(site);
            Map<String, Map<String, AlarmDetails>> alarmDetailsMap = alarmDetailConfigService.loadAlarmDetailsFromRedisOrDB(siteDetailsDTO.getSiteId());

            Map<String, AlarmDetails> alarmMap = alarmDetailsMap.get(site);
            List<AlarmDetailConfig> alarmDetailConfigList = finalAlarmConfigMap.values().stream().filter(p -> Objects.equals(p.getContractId(), finalContractId)).toList();

            if (alarmMap != null && !alarmMap.values().isEmpty()) {
                alarmMap.values().stream()
                        .forEach(alarm -> {
                            Optional<AlarmDetailConfig> alarmDetailConfig = alarmDetailConfigList.stream().filter(p -> Util.removeUnderscoreWithSpace(p.getAlarmName()).equals(alarm.getAlarmName())).findFirst();
                            alarmDetails.add(new AlarmDetailReportDto(alarm, siteDetailsDTO, alarmDetailConfig));
                        });
                siteWiseAlarmMap.put(site, alarmDetails);
            }
        });
        siteWiseAlarmMap.entrySet().forEach(p -> {
            resultList.addAll(p.getValue().stream().sorted(Comparator.comparing(AlarmDetailReportDto::getFromTime)).collect(Collectors.toList()));
        });

        return resultList;
    }
/*    private List<AlarmDetailReportDto> getContractAlarms2(Integer finalContractId, List<Integer> operatorId) {
        List<String> siteList = getSiteList(finalContractId);
        List<AlarmDetailReportDto> resultList = new ArrayList<>();
        Map<String, List<AlarmDetailReportDto>> siteWiseAlarmMap = new HashMap<>();

        siteList.forEach(site -> {
            List<AlarmDetailReportDto> alarmDetails = new ArrayList<>();
            SiteDetailsDTO siteDetailsDTO = rmsCoreClientService.getSiteDetailsBySiteId(site);
            Map<String, AlarmDetails> alarmMap = Constants.alarmDetailMap.get(site);
            List<AlarmDetailConfig> alarmDetailConfigList = Constants.alarmConfigMap.values()
                    .stream()
                    .filter(p -> p.getContractId() == finalContractId)
                    .collect(Collectors.toList());

            // Fetch site-wise alarm masking config
            Map<String, SiteWiseAlarmMaskingConfig> alarmMaskingConfigMap = siteWiseAlarmMaskingConfigService.getAlarmMaskingConfigMap(site);

            if (alarmMap != null && !alarmMap.values().isEmpty()) {
                alarmMap.values().stream()
                        .filter(alarm -> {
                            SiteWiseAlarmMaskingConfig config = alarmMaskingConfigMap.get(alarm.getAlarmName());
                            return config == null || config.getAlarmEnabled(); // Include only if not in config or explicitly enabled
                        })
                        .forEach(alarm -> {
                            Optional<AlarmDetailConfig> alarmDetailConfig = alarmDetailConfigList.stream()
                                    .filter(p -> Util.removeUnderscoreWithSpace(p.getAlarmName()).equals(alarm.getAlarmName()))
                                    .findFirst();
                            alarmDetails.add(new AlarmDetailReportDto(alarm, siteDetailsDTO, alarmDetailConfig));
                        });

                siteWiseAlarmMap.put(site, alarmDetails);
            }
        });

        siteWiseAlarmMap.forEach((site, alarms) -> {
            resultList.addAll(alarms.stream().sorted(Comparator.comparing(AlarmDetailReportDto::getFromTime)).collect(Collectors.toList()));
        });

        return resultList;
    }*/


    private List<String> getSiteList(Integer finalContractId) {
        List<String> siteList = commonService.getAllProtocolSites().stream()
                .filter(p -> p.getContractParentId().equals(finalContractId))
                .map(site -> site.getSiteId())
                .collect(Collectors.toList());
        return siteList;
    }


}
