package com.ieng.rms.cntroller;

import com.ieng.rms.dto.AlarmMaskingDTO;
import com.ieng.rms.services.AlarmDetailConfigService;
import com.ieng.rms.services.SiteWiseAlarmMaskingConfigService;
import com.ieng.rms.util.Constants;
import com.ieng.rms.util.RedisUtil;
import com.ieng.rms.util.URL;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@CrossOrigin("*")
@RestController
@RequestMapping(URL.DEFAULT + URL.ALARMS_MASKING)
public class SiteWiseAlarmMaskingConfigController {

    private final RedisUtil redisUtil;

    private final SiteWiseAlarmMaskingConfigService siteWiseAlarmMaskingConfigService;
    private final AlarmDetailConfigService alarmDetailConfigService;

    public SiteWiseAlarmMaskingConfigController(RedisUtil redisUtil, SiteWiseAlarmMaskingConfigService siteWiseAlarmMaskingConfigService, AlarmDetailConfigService alarmDetailConfigService) {
        this.redisUtil = redisUtil;
        this.siteWiseAlarmMaskingConfigService = siteWiseAlarmMaskingConfigService;
        this.alarmDetailConfigService = alarmDetailConfigService;
    }

    @GetMapping("/allAlarmsMasking")
    public List<AlarmMaskingDTO> getAllConfigs() {
        return siteWiseAlarmMaskingConfigService.getAllAlarmMaskingData();
    }

    @GetMapping("/configBySiteId")
    public AlarmMaskingDTO getConfigBySiteId(@RequestParam String siteId) {
        return siteWiseAlarmMaskingConfigService.getConfigsBySiteId(siteId);
    }

    @PostMapping("/saveAlarmMasking")
    public ResponseEntity<?> createConfig(@RequestBody AlarmMaskingDTO alarmMaskingDTO) {
        return siteWiseAlarmMaskingConfigService.saveConfig(alarmMaskingDTO);
    }

    @PostMapping("/updateAlarmMasking")
    public ResponseEntity<?> updateConfig(@RequestBody AlarmMaskingDTO alarmMaskingDTO) {
        return siteWiseAlarmMaskingConfigService.updateConfig(alarmMaskingDTO);
    }

    @PostMapping("/deleteAlarmMasking")
    public ResponseEntity<?> deleteConfig(@RequestParam List<Long> alarmMaskingIds) {
        return siteWiseAlarmMaskingConfigService.deleteAlarms(alarmMaskingIds);
    }

    @GetMapping("/alarmMasking/escalationEnable")
    public void getEnableAlarmsMasking() {
        siteWiseAlarmMaskingConfigService.getAllAlarmMaskingEscalationEnable();
    }

    @DeleteMapping("/clear-cache")
    public ResponseEntity<String> clearAllAlarmConfigCache() {
        Constants.alarmsMaskingMap.clear();
        return ResponseEntity.ok("All alarm configurations cache cleared.");
    }

    @DeleteMapping("/clearConfigFromRedis/{siteId}")
    public ResponseEntity<?> clearConfigForSite(@PathVariable String siteId) {
        boolean deleted = redisUtil.deleteAlarmMaskingConfig(siteId);
        if (deleted) {
            return ResponseEntity.ok(Map.of("message", "Config map for site " + siteId + " cleared successfully."));
        } else {
            return ResponseEntity.status(HttpStatus.NOT_FOUND)
                    .body(Map.of("errors", "No config map found for site " + siteId + " or deletion failed."));
        }
    }

}
