package com.ieng.rms.config;

import org.apache.activemq.ActiveMQConnectionFactory;
import org.apache.activemq.command.ActiveMQQueue;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.DependsOn;
import org.springframework.context.annotation.Primary;
import org.springframework.jms.annotation.EnableJms;
import org.springframework.jms.config.DefaultJmsListenerContainerFactory;
import org.springframework.jms.config.JmsListenerContainerFactory;
import org.springframework.jms.core.JmsMessagingTemplate;

import java.util.ArrayList;


@Configuration
@EnableJms
@DependsOn("prerequisiteLoader")
public class AMQConfiguration {

    @Value("${jsa.activemq.monitor-alarms}")
    public String sonicAlarmQueue;

    @Value("${spring.activemq.broker-url}")
    private String brokerUrl;

    @Value("${spring.activemq.user}")
    private String userName;

    @Value("${spring.activemq.password}")
    private String password;


    @Bean
    public JmsListenerContainerFactory<?> jmsListenerContainerQueue(ActiveMQConnectionFactory connectionFactory) {
        DefaultJmsListenerContainerFactory bean = new DefaultJmsListenerContainerFactory();
        bean.setConnectionFactory(connectionFactory);
        return bean;
    }

    @Bean
    public JmsListenerContainerFactory<?> jmsListenerContainerTopicNotification(ActiveMQConnectionFactory connectionFactory) {
        DefaultJmsListenerContainerFactory bean = new DefaultJmsListenerContainerFactory();

        bean.setPubSubDomain(true);
        bean.setConnectionFactory(connectionFactory);
        bean.setRecoveryInterval(1000L);
        return bean;
    }

    @Bean
    public ActiveMQConnectionFactory connectionFactory() {
        ActiveMQConnectionFactory factory = new ActiveMQConnectionFactory();
        factory.setBrokerURL(brokerUrl);
        factory.setUserName(userName);
        factory.setPassword(password);
        factory.setTrustAllPackages(new ArrayList().add("com.ieng.rms.entity"));
        return factory;
    }

    @Bean
    @Primary
    public JmsMessagingTemplate jmsMessagingTemplate() {

        JmsMessagingTemplate jmsMessagingTemplate = new JmsMessagingTemplate(connectionFactory());
        jmsMessagingTemplate.setDefaultDestination(alarmQueue());

        return jmsMessagingTemplate;
    }

    @Bean
    ActiveMQQueue alarmQueue() {
        return new ActiveMQQueue(sonicAlarmQueue);
    }

}
