package com.ieng.rms.models;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.ieng.rms.custom.RoundUpto0;
import com.ieng.rms.custom.RoundUpto1;
import com.ieng.rms.custom.RoundUpto2;
import com.ieng.rms.dto.AlarmConfigDto;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.List;

@Data
@Transactional
@NoArgsConstructor
@AllArgsConstructor
public class AlarmsModel implements Serializable {
    private String lastUpdated;
    private Integer packetId;
    private String siteId;
    private String siteName;
    private String energyModel;
    private String batteryDischarge;
    private String batteryFuseFail;
    private String blvdAlarm;
    private String dcVoltageLow;
    private String dgBatteryLow;
    private String dgFailToStart;
    private String dgManualMode;
    private String doorOpenAlarm;
    private String loadBreakerFail;
    private String loadOnDG;
    private String loadOnSolar;
    private String lowFuelLevel;
    private String mainsFailure;
    private String mainsOnBatteryDischarge;
    private String rectifierCommunicationFailure;
    private String rectifierFail;
    private String siteOnBattery;
    private String smokeAndFire;
    private String systemBatterytemp4HighAlarm;
    private String temperatureSensorLoseorFaultAlarm;
    private String solarControllerFail;
    private String highTemperature;
    private String region;
    private String zone;
    private String cluster;
    private String solarHybridMode;
    private String clientSiteId;
    private String fanFailDI1;
    private String doorOpenDI2;
    private String smokeFireDI3;
    private String dI4Spare;
    private String dI5Spare;
    private String dcVoltageHigh;

    @JsonSerialize(using = RoundUpto1.class)
    private float dcV;
    private Integer fuel;
    private String powerSource;
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "##")
    private String soc;
    @JsonSerialize(using = RoundUpto2.class)
    private float dcKW;

    private Integer contractParent;

    private long lastUpdate;
    private boolean isSolar = false;
    private List<String> tickersList = null;

    private String rectifierControllerCommFault;
    private String acemCommFault;
    private String dcemCommFault;
    private String gcuCommFault;
    private String solarEmCommFault;
    private String fuelSensorCommFault;

    // Sonic V2 Live Parameters
    private String gpsLatitude;
    private String gpsLongitude;
    private Integer wifiStatus;
    private String packetType;
    @JsonSerialize(using = RoundUpto1.class)
    private float totalDcLoadCurrent;
    @JsonSerialize(using = RoundUpto1.class)
    private double dcLoadPowerKW;

    private Integer batteryCapacity;
    private Integer batteryCurrent;
    @JsonSerialize(using = RoundUpto0.class)
    private float batteryRunhours;
    @JsonSerialize(using = RoundUpto0.class)
    private float batteryDischargeEnergy;

    @JsonSerialize(using = RoundUpto0.class)
    private double dgPowerKW;
    @JsonSerialize(using = RoundUpto0.class)
    private float dgEnergy;
    @JsonSerialize(using = RoundUpto0.class)
    private float dgRunhours;
    @JsonSerialize(using = RoundUpto1.class)
    private float dgBatteryVoltage;
    @JsonSerialize(using = RoundUpto1.class)
    private double gridPowerKW;
    @JsonSerialize(using = RoundUpto0.class)
    private float gridEnergy;
    @JsonSerialize(using = RoundUpto0.class)
    private float gridRunhours;
    @JsonSerialize(using = RoundUpto1.class)
    private double pvTotalCurrent;
    @JsonSerialize(using = RoundUpto1.class)
    private double pvTotalPowerInKW;
    @JsonSerialize(using = RoundUpto0.class)
    private float pvEnergy;
    @JsonSerialize(using = RoundUpto0.class)
    private float pvRunhours;
    @JsonSerialize(using = RoundUpto0.class)
    private float shelterTemperature;
    @JsonSerialize(using = RoundUpto2.class)
    private float ambientTemperature;
    @JsonSerialize(using = RoundUpto1.class)
    private float batteryCabinetTemperature;
    private String systemControllerFirmwareVersion;
    private String rmsFirmwareVersion;

    // Sonic V2 Live Alarms
    private String batteryFault;
    private String rectifierModuleFail;
    private String pvModuleFail;
    private String dgCommonFault;
    private String surgeProtectionDeviceFail;
    private String liCommFault;
    private String dcAirCon1CommFault;
    private String dcAirCon2CommFault;
    private String dgEmergencyStop;
    private String dgOverLoad;
    private String dgOnWithoutLoud;
    private String dgFailToStop;
    private String dgLlop;
    private String dgHwt;
    private String gpsFixed;
    private String fanFail;
    private String systemInManualMode;
    private String tamperSwitch;
    // private String dcHvModuleDisconnect;
    //private String highTemperatureModuleShutdown;
    private String blvdTrip;
    private String llvdTrip;
    private String dcAirConCompressorFanFault;
    private String singleRectifierFault;
    private String multipleRectifierFault;
    private String fuelAdditionAlarm;
    private String fuelPilferageAlarm;

    // Tanent/Operator Id from packet
    private String clientId;
    private String tenantName;

    private List<AlarmConfigDto> alarmConfigMap;
    private String aCSurgeProtectionDeviceFail;
    private String dCSurgeProtectionDeviceFail;
    private String airConFail;
    private String ebFail;

    @JsonIgnore
    private String moduleSerialNumber1;
    @JsonIgnore
    private String moduleSerialNumber2;
    @JsonIgnore
    private String moduleSerialNumber3;
    @JsonIgnore
    private String moduleSerialNumber4;
    @JsonIgnore
    private String moduleSerialNumber5;
    @JsonIgnore
    private String moduleSerialNumber6;
    @JsonIgnore
    private String moduleSerialNumber7;
    @JsonIgnore
    private String moduleSerialNumber8;
    @JsonIgnore
    private String moduleSerialNumber9;
    @JsonIgnore
    private String moduleSerialNumber10;
    @JsonIgnore
    private String moduleSerialNumber11;
    @JsonIgnore
    private String moduleSerialNumber12;
    @JsonIgnore
    private String moduleSerialNumber13;
    @JsonIgnore
    private String moduleSerialNumber14;
    @JsonIgnore
    private String moduleSerialNumber15;
    @JsonIgnore
    private String moduleSerialNumber16;
    @JsonIgnore
    private String moduleSerialNumber17;
    @JsonIgnore
    private String moduleSerialNumber18;
    @JsonIgnore
    private String moduleSerialNumber19;
    @JsonIgnore
    private String moduleSerialNumber20;

    private String socBattery1;

}
