package com.ieng.rms.repository;

import com.ieng.rms.entity.AlarmDetails;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;
import reactor.core.publisher.Flux;

import java.time.LocalDateTime;
import java.util.List;

@Repository
public interface AlarmDetailRepository extends JpaRepository<AlarmDetails, Integer> {

    @Query(" from AlarmDetails ad where ad.alarmName in (:alarmName) and ad.siteId = :siteId and ad.alarmStatus = 1")
    public List<AlarmDetails> findAlarmDetail(@Param("alarmName") List<String> alarmName, @Param("siteId") String siteId);

    @Query(" from AlarmDetails ad where ad.siteId = :siteId and ad.toTime is null")
    public List<AlarmDetails> findOpenAlarms(@Param("siteId") String siteId);

    @Query(value = " Select * from (select * from alarm_details ad where ad.from_time <= :to) as alarms where (alarms.from_time >= :from or alarms.to_time is null ) and alarms.from_time <= :to  and alarms.site_id in (:siteIds)",nativeQuery = true)
    public List<AlarmDetails> findAlarms(@Param("from") LocalDateTime from, @Param("to") LocalDateTime toTime, List<String> siteIds);

    @Query(value = " Select * from (select * from alarm_details ad where ad.from_time <= :to) as alarms where (alarms.from_time >= :from or alarms.to_time is null ) and alarms.from_time <= :to  and alarms.site_id in (:siteIds) and alarms.alarm_name in (:alarmNames)",nativeQuery = true)
    public List<AlarmDetails> findAlarmsByAlarmParams(@Param("from") LocalDateTime from, @Param("to") LocalDateTime toTime, List<String> siteIds, List<String> alarmNames);

    @Query(" from AlarmDetails ad where ad.alarmName = :name order by ad.id desc ")
    public List<AlarmDetails> findByAlarmName(@Param("name") String alarmName);

    @Query(" from AlarmDetails ad where (ad.fromTime >= :from and ad.fromTime <= :to) and ad.siteId = :siteId ")
    public List<AlarmDetails> findAlarmsBySiteId(@Param("from") LocalDateTime from, @Param("to") LocalDateTime toTime, @Param("siteId") String siteId);

    @Query(" from AlarmDetails ad where (ad.fromTime >= :from and ad.fromTime <= :to) and ad.siteId in (:siteIds) and ad.alarmName = :alarmName")
    public List<AlarmDetails> findByAlarmNameAndSite(LocalDateTime from, LocalDateTime to, List<String> siteIds, String alarmName);

    @Query(" from AlarmDetails ad where (ad.fromTime >= :from and ad.fromTime <= :to) and ad.clientId in (:clientIds) and ad.alarmName = :alarmName")
    public List<AlarmDetails> findByAlarmNameAndClient(LocalDateTime from, LocalDateTime to, List<String> clientIds, String alarmName);


}