package com.ieng.rms.services;

import com.ieng.rms.entity.SiteWiseAlarmMaskingConfig;
import com.ieng.rms.models.MonitorAlarmModel;
import com.ieng.rms.models.MonitorScreenModel;
import com.ieng.rms.models.PacketModel;
import com.ieng.rms.models.SiteDetailsDTO;
import com.ieng.rms.repository.AlarmDetailConfigRepository;
import com.ieng.rms.util.Util;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class MonitorAlarmsService {

    private final AlarmDetailConfigRepository alarmDetailConfigRepository;

    private final SiteWiseAlarmMaskingConfigService siteWiseAlarmMaskingConfigService;

    private final AlarmDetailConfigService alarmDetailConfigService;

    public MonitorAlarmsService(AlarmDetailConfigRepository alarmDetailConfigRepository, SiteWiseAlarmMaskingConfigService siteWiseAlarmMaskingConfigService, AlarmDetailConfigService alarmDetailConfigService) {
        this.alarmDetailConfigRepository = alarmDetailConfigRepository;
        this.siteWiseAlarmMaskingConfigService = siteWiseAlarmMaskingConfigService;
        this.alarmDetailConfigService = alarmDetailConfigService;
    }

    //power sources and energymodel
    public MonitorAlarmModel initMonitor(PacketModel model, SiteDetailsDTO site, String energyModel) {
        MonitorAlarmModel monitor = new MonitorAlarmModel();
        Map<String, String> alarmCondifMap = alarmDetailConfigService.getAlarmConfigMapByContractId(site.getContractParentId());
        MonitorScreenModel monitorScreenModel = new MonitorScreenModel(model, site, energyModel, alarmCondifMap);
        monitor.setAlarmConfigMap(alarmDetailConfigRepository.getAlarmDetailConfigList(site.getContractParentId()));

        monitor.setEnergyModel((energyModel != null) ? energyModel : "N/A");
        monitor.setPowerSource(monitorScreenModel.getPowerSource());

        monitor.setPacketId(model.getPacketId());
        monitor.setClientId(model.getClientId());
        monitor.setSiteId(site.getSiteId());
        monitor.setSiteName(site.getSiteName());
        if (site.getRegionName() != null) {
            monitor.setRegion(site.getRegionName());
        }

//			this.zone = sit;
        if (site.getClusterName() != null) {
            monitor.setCluster(site.getClusterName());
        } else {
            monitor.setCluster(null);
        }
        monitor.setClientSiteId(site.getSiteId());

        if (site.getContractParentId() != null)
            monitor.setContractParent(site.getContractParentId());

        monitor.setTickersList(new ArrayList<>());
        monitor.setLastUpdated(model.getDateAndTimeStamp().toString());
        monitor.setLastUpdate(model.getDateAndTimeStamp().getTime());

        Map<String, String> alarmStatusMap = new HashMap<>();

        List<String> alarmsToCheck = getAllAlarms();

// Check alarm status for each alarm
        alarmsToCheck.forEach(alarmName -> {
            String alarmStatus = getAlarmStatusFromConfig(monitor.getSiteId(), alarmName);
            alarmStatusMap.put(alarmName, alarmStatus);
        });

        if (alarmStatusMap.get(Util.AlarmNames.BATTERY_ON_DISCHARGE) == null)
            monitor.setBatteryDischarge(
                    (model.isBatteryOnDischarge() == 1)
                            ? String.valueOf(Util.getAlarmCriticality(Util.AlarmNames.BATTERY_ON_DISCHARGE, monitor.getAlarmConfigMap()))
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        else monitor.setBatteryDischarge(
                alarmStatusMap.get(Util.AlarmNames.BATTERY_ON_DISCHARGE)
        );
        // Battery Fuse Fail Alarm
        if (alarmStatusMap.get(Util.AlarmNames.BATTERY_FUSE_FAIL) == null) {
            monitor.setBatteryFuseFail(
                    (model.isBatteryFuseFail() == 1)
                            ? Util.getAlarmCriticality(Util.AlarmNames.BATTERY_FUSE_FAIL, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setBatteryFuseFail(alarmStatusMap.get(Util.AlarmNames.BATTERY_FUSE_FAIL));
        }

// LLVD OR LVBD Trip Alarm
        if (alarmStatusMap.get(Util.AlarmNames.LLVD_OR_LVBD_TRIP) == null) {
            monitor.setBlvdAlarm(
                    (model.isdI10Spare() == 1)
                            ? Util.getAlarmCriticality(Util.AlarmNames.LLVD_OR_LVBD_TRIP, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setBlvdAlarm(alarmStatusMap.get(Util.AlarmNames.LLVD_OR_LVBD_TRIP));
        }

// DC Voltage Low Alarm
        if (alarmStatusMap.get(Util.AlarmNames.DC_VOLTAGE_LOW) == null) {
            monitor.setDcVoltageLow(
                    (model.isdCVoltageLow() == 1)
                            ? Util.getAlarmCriticality(Util.AlarmNames.DC_VOLTAGE_LOW, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDcVoltageLow(alarmStatusMap.get(Util.AlarmNames.DC_VOLTAGE_LOW));
        }

// DG Battery Voltage Low Alarm
        if (alarmStatusMap.get(Util.AlarmNames.DG_BATTERY_VOLTAGE_LOW) == null) {
            monitor.setDgBatteryLow(
                    (model.isdGBatteryVoltageLow() == 1)
                            ? Util.getAlarmCriticality(Util.AlarmNames.DG_BATTERY_VOLTAGE_LOW, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDgBatteryLow(alarmStatusMap.get(Util.AlarmNames.DG_BATTERY_VOLTAGE_LOW));
        }

// DG Fail To Start Alarm
        if (alarmStatusMap.get(Util.AlarmNames.DG_FAIL_TO_START) == null) {
            monitor.setDgFailToStart(
                    (model.isdGFailToStart() == 1)
                            ? Util.getAlarmCriticality(Util.AlarmNames.DG_FAIL_TO_START, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDgFailToStart(alarmStatusMap.get(Util.AlarmNames.DG_FAIL_TO_START));
        }

// DG Running in Manual Mode Alarm
        if (alarmStatusMap.get(Util.AlarmNames.DG_RUNNING_IN_MANUAL_MODE) == null) {
            monitor.setDgManualMode(
                    (model.isdG12OnInManualMode() == 1)
                            ? Util.getAlarmCriticality(Util.AlarmNames.DG_RUNNING_IN_MANUAL_MODE, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDgManualMode(alarmStatusMap.get(Util.AlarmNames.DG_RUNNING_IN_MANUAL_MODE));
        }

// Load Breaker Tripped Alarm
        if (alarmStatusMap.get(Util.AlarmNames.LOAD_BREAKER_TRIPPED) == null) {
            monitor.setLoadBreakerFail(
                    (model.isLoadBreakerTripped() == 1)
                            ? Util.getAlarmCriticality(Util.AlarmNames.LOAD_BREAKER_TRIPPED, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setLoadBreakerFail(alarmStatusMap.get(Util.AlarmNames.LOAD_BREAKER_TRIPPED));
        }

// Mains Failure Alarm
        if (alarmStatusMap.get(Util.AlarmNames.MAINS_FAIL) == null) {
            monitor.setMainsFailure(
                    (model.isMainsFail() == 1)
                            ? Util.getAlarmCriticality(Util.AlarmNames.MAINS_FAIL, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setMainsFailure(alarmStatusMap.get(Util.AlarmNames.MAINS_FAIL));
        }

// Battery High Temperature Alarm
        if (alarmStatusMap.get(Util.AlarmNames.BATTER_TEMP_HIGH) == null) {
            monitor.setHighTemperature(
                    (model.isBatterTempHigh() == 1)
                            ? Util.getAlarmCriticality(Util.AlarmNames.BATTER_TEMP_HIGH, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setHighTemperature(alarmStatusMap.get(Util.AlarmNames.BATTER_TEMP_HIGH));
        }

// Rectifier Communication Failure Alarm
        if (alarmStatusMap.get(Util.AlarmNames.RECTIFIER_CONTROLLER_COMMUNICATION_FAULT) == null) {
            monitor.setRectifierCommunicationFailure(
                    (model.isRectifierControllerCommFault() != null && model.isRectifierControllerCommFault() == 1)
                            ? Util.getAlarmCriticality(Util.AlarmNames.RECTIFIER_CONTROLLER_COMMUNICATION_FAULT, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setRectifierCommunicationFailure(alarmStatusMap.get(Util.AlarmNames.RECTIFIER_CONTROLLER_COMMUNICATION_FAULT));
        }

// Door Open Alarm
        if (alarmStatusMap.get(Util.AlarmNames.DOOR_OPEN) == null) {
            monitor.setDoorOpenAlarm(
                    (model.isDoorOpenDI2() == 1)
                            ? Util.getAlarmCriticality(Util.AlarmNames.DOOR_OPEN, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDoorOpenAlarm(alarmStatusMap.get(Util.AlarmNames.DOOR_OPEN));
        }

// Battery Discharge Alarm
        if (alarmStatusMap.get(Util.AlarmNames.BATTERY_ON_DISCHARGE) == null) {
            monitor.setBatteryDischarge(
                    (model.isBatteryOnDischarge() == 1)
                            ? String.valueOf(Util.getAlarmCriticality(Util.AlarmNames.BATTERY_ON_DISCHARGE, monitor.getAlarmConfigMap()))
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setBatteryDischarge(alarmStatusMap.get(Util.AlarmNames.BATTERY_ON_DISCHARGE));
        }

// RMS Firmware Version
        monitor.setRmsFirmwareVersion(model.getSonicFirmwareVersion());


        if (alarmStatusMap.get(Util.AlarmNames.DOOR_OPEN) == null) {
            monitor.setDoorOpenAlarm(
                    (model.isDoorOpenDI2() == 1)
                            ? Util.getAlarmCriticality(Util.AlarmNames.DOOR_OPEN, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDoorOpenAlarm(alarmStatusMap.get(Util.AlarmNames.DOOR_OPEN));
        }


        if (Util.checkSonicVersion(model.getSonicFirmwareVersion())) {
            if (alarmStatusMap.get(Util.AlarmNames.SYSTEM_BATTERY_TEMP4_HIGH) == null) {
                monitor.setSystemBatterytemp4HighAlarm(
                        model.getSystemBatterytemp4High().equals("1")
                                ? Util.getAlarmCriticality(Util.AlarmNames.SYSTEM_BATTERY_TEMP4_HIGH, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setSystemBatterytemp4HighAlarm(alarmStatusMap.get(Util.AlarmNames.SYSTEM_BATTERY_TEMP4_HIGH));
            }

            if (alarmStatusMap.get(Util.AlarmNames.TEMPERATURE_SENSOR_LOSE_OR_FAULT_ALARM) == null) {
                monitor.setTemperatureSensorLoseorFaultAlarm(
                        model.getTemperatureSensorLoseorFaultAlarm().equals("1")
                                ? Util.getAlarmCriticality(Util.AlarmNames.TEMPERATURE_SENSOR_LOSE_OR_FAULT_ALARM, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setTemperatureSensorLoseorFaultAlarm(alarmStatusMap.get(Util.AlarmNames.TEMPERATURE_SENSOR_LOSE_OR_FAULT_ALARM));
            }

            if (alarmStatusMap.get(Util.AlarmNames.FAN_FailDI1) == null) {
                monitor.setFanFailDI1(
                        model.getFanFailDI1() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.FAN_FailDI1, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setFanFailDI1(alarmStatusMap.get(Util.AlarmNames.FAN_FailDI1));
            }

            if (alarmStatusMap.get(Util.AlarmNames.DOOR_OPENDI2) == null) {
                monitor.setDoorOpenDI2(
                        model.getDoorOpenDI2() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.DOOR_OPENDI2, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setDoorOpenDI2(alarmStatusMap.get(Util.AlarmNames.DOOR_OPENDI2));
            }

            if (alarmStatusMap.get(Util.AlarmNames.SMOKE_FIREDI3) == null) {
                monitor.setSmokeFireDI3(
                        model.getSmokeFireDI3() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.SMOKE_FIREDI3, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setSmokeFireDI3(alarmStatusMap.get(Util.AlarmNames.SMOKE_FIREDI3));
            }

// Apply the same check for all remaining alarms
            if (alarmStatusMap.get(Util.AlarmNames.SPARE_DI4) == null) {
                monitor.setDI4Spare(
                        model.getdI4Spare() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.SPARE_DI4, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setDI4Spare(alarmStatusMap.get(Util.AlarmNames.SPARE_DI4));
            }

            if (alarmStatusMap.get(Util.AlarmNames.SPARE_DI5) == null) {
                monitor.setDI5Spare(
                        model.getdI5Spare() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.SPARE_DI5, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setDI5Spare(alarmStatusMap.get(Util.AlarmNames.SPARE_DI5));
            }

            if (alarmStatusMap.get(Util.AlarmNames.SPARE_DI6) == null) {
                monitor.setDI6Spare(
                        model.getdI6Spare() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.SPARE_DI6, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setDI6Spare(alarmStatusMap.get(Util.AlarmNames.SPARE_DI6));
            }

            if (alarmStatusMap.get(Util.AlarmNames.SPARE_DI7) == null) {
                monitor.setDI7Spare(
                        model.getdI7Spare() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.SPARE_DI7, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setDI7Spare(alarmStatusMap.get(Util.AlarmNames.SPARE_DI7));
            }

            if (alarmStatusMap.get(Util.AlarmNames.SPARE_DI8) == null) {
                monitor.setDI8Spare(
                        model.getdI8Spare() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.SPARE_DI8, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setDI8Spare(alarmStatusMap.get(Util.AlarmNames.SPARE_DI8));
            }

            if (alarmStatusMap.get(Util.AlarmNames.SPARE_DI9) == null) {
                monitor.setDI9Spare(
                        model.getdI9Spare() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.SPARE_DI9, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setDI9Spare(alarmStatusMap.get(Util.AlarmNames.SPARE_DI9));
            }

            if (alarmStatusMap.get(Util.AlarmNames.SPARE_DI10) == null) {
                monitor.setDI10Spare(
                        model.getdI10Spare() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.SPARE_DI10, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setDI10Spare(alarmStatusMap.get(Util.AlarmNames.SPARE_DI10));
            }

            if (alarmStatusMap.get(Util.AlarmNames.SPARE_DI11) == null) {
                monitor.setDI11Spare(
                        model.getdI11Spare() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.SPARE_DI11, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setDI11Spare(alarmStatusMap.get(Util.AlarmNames.SPARE_DI11));
            }

            if (alarmStatusMap.get(Util.AlarmNames.SPARE_DI12) == null) {
                monitor.setDI12Spare(
                        model.getdI12Spare() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.SPARE_DI12, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setDI12Spare(alarmStatusMap.get(Util.AlarmNames.SPARE_DI12));
            }

            if (alarmStatusMap.get(Util.AlarmNames.SPARE_DI13) == null) {
                monitor.setSonicDI13(
                        model.getSonicDI13() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.SPARE_DI13, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setSonicDI13(alarmStatusMap.get(Util.AlarmNames.SPARE_DI13));
            }

            if (alarmStatusMap.get(Util.AlarmNames.SPARE_DI14) == null) {
                monitor.setSonicDI14(
                        model.getSonicDI14() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.SPARE_DI14, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setSonicDI14(alarmStatusMap.get(Util.AlarmNames.SPARE_DI14));
            }

            if (alarmStatusMap.get(Util.AlarmNames.SPARE_DI15) == null) {
                monitor.setSonicDI15(
                        model.getSonicDI15() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.SPARE_DI15, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setSonicDI15(alarmStatusMap.get(Util.AlarmNames.SPARE_DI15));
            }

            if (alarmStatusMap.get(Util.AlarmNames.SPARE_DI16) == null) {
                monitor.setSonicDI16(
                        model.getSonicDI16() == 1
                                ? Util.getAlarmCriticality(Util.AlarmNames.SPARE_DI16, monitor.getAlarmConfigMap())
                                : Util.AlarmStatus.NORMAL_ALARM
                );
            } else {
                monitor.setSonicDI16(alarmStatusMap.get(Util.AlarmNames.SPARE_DI16));
            }

        }

        if (alarmStatusMap.get(Util.AlarmNames.LOAD_ON_DG) == null) {
            monitor.setLoadOnDG(
                    model.isLoadOnDG12() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.LOAD_ON_DG, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setLoadOnDG(alarmStatusMap.get(Util.AlarmNames.LOAD_ON_DG));
        }

        if (alarmStatusMap.get(Util.AlarmNames.LOAD_ON_SOLAR) == null) {
            if (model.getReserved2LoadOnPv() != null && model.getReserved2LoadOnPv() == 1) {
                monitor.setSolar(true);
                monitor.setLoadOnSolar(Util.getAlarmCriticality(Util.AlarmNames.LOAD_ON_SOLAR, monitor.getAlarmConfigMap()));
            } else {
                monitor.setSolar(false);
                monitor.setLoadOnSolar(Util.AlarmStatus.NORMAL_ALARM);
            }
        } else {
            monitor.setLoadOnSolar(alarmStatusMap.get(Util.AlarmNames.LOAD_ON_SOLAR));
        }

        if (alarmStatusMap.get(Util.AlarmNames.DG_LOW_FUEL_LEVEL) == null) {
            monitor.setLowFuelLevel(
                    model.isdGLowFuelLevel() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.DG_LOW_FUEL_LEVEL, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setLowFuelLevel(alarmStatusMap.get(Util.AlarmNames.DG_LOW_FUEL_LEVEL));
        }

        if (alarmStatusMap.get(Util.AlarmNames.SITE_ON_BATTERY) == null) {
            monitor.setSiteOnBattery(
                    model.isLoadOnBattery() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.SITE_ON_BATTERY, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setSiteOnBattery(alarmStatusMap.get(Util.AlarmNames.SITE_ON_BATTERY));
        }

        if (alarmStatusMap.get(Util.AlarmNames.SMOKE_FIRE) == null) {
            monitor.setSmokeAndFire(
                    model.isSmokeFireDI3() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.SMOKE_FIRE, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setSmokeAndFire(alarmStatusMap.get(Util.AlarmNames.SMOKE_FIRE));
        }

        if (alarmStatusMap.get(Util.AlarmNames.SOLAR_HYBRID_MODE) == null) {
            monitor.setSolarHybridMode(
                    model.getReserved3PvHybridMode() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.SOLAR_HYBRID_MODE, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setSolarHybridMode(alarmStatusMap.get(Util.AlarmNames.SOLAR_HYBRID_MODE));
        }

        if (alarmStatusMap.get(Util.AlarmNames.SOLAR_CONTROLLER_FAIL) == null) {
            monitor.setSolarControllerFail(
                    model.isSolarEMCOMMfault() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.SOLAR_CONTROLLER_FAIL, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setSolarControllerFail(alarmStatusMap.get(Util.AlarmNames.SOLAR_CONTROLLER_FAIL));
        }

        if (alarmStatusMap.get(Util.AlarmNames.DC_VOLTAGE_HIGH) == null) {
            monitor.setDcVoltageHigh(
                    model.getDcVoltageHigh() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.DC_VOLTAGE_HIGH, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDcVoltageHigh(alarmStatusMap.get(Util.AlarmNames.DC_VOLTAGE_HIGH));
        }

        if (alarmStatusMap.get(Util.AlarmNames.RECTIFIER_CONTROLLER_COMMUNICATION_FAULT) == null) {
            monitor.setRectifierControllerCommFault(
                    model.isRectifierControllerCommFault() != null && model.isRectifierControllerCommFault() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.RECTIFIER_CONTROLLER_COMMUNICATION_FAULT, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setRectifierControllerCommFault(alarmStatusMap.get(Util.AlarmNames.RECTIFIER_CONTROLLER_COMMUNICATION_FAULT));
        }

        if (alarmStatusMap.get(Util.AlarmNames.ACEM_COMM_FAULT) == null) {
            monitor.setAcemCommFault(
                    model.getaCEMCommFault() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.ACEM_COMM_FAULT, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setAcemCommFault(alarmStatusMap.get(Util.AlarmNames.ACEM_COMM_FAULT));
        }

        if (alarmStatusMap.get(Util.AlarmNames.DCEM_COMM_FAULT) == null) {
            monitor.setDcemCommFault(
                    model.isdCEMCOMMFault() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.DCEM_COMM_FAULT, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDcemCommFault(alarmStatusMap.get(Util.AlarmNames.DCEM_COMM_FAULT));
        }

        if (alarmStatusMap.get(Util.AlarmNames.GCU_COMM_FAULT) == null) {
            monitor.setGcuCommFault(
                    model.isgCUCommFault() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.GCU_COMM_FAULT, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setGcuCommFault(alarmStatusMap.get(Util.AlarmNames.GCU_COMM_FAULT));
        }

        if (alarmStatusMap.get(Util.AlarmNames.PV_EM_COMM_FAULT) == null) {
            monitor.setSolarEmCommFault(
                    model.isSolarEMCOMMfault() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.PV_EM_COMM_FAULT, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setSolarEmCommFault(alarmStatusMap.get(Util.AlarmNames.PV_EM_COMM_FAULT));
        }

        if (alarmStatusMap.get(Util.AlarmNames.FUEL_SENSOR_COMM_FAULT) == null) {
            monitor.setFuelSensorCommFault(
                    model.isFuelSensorCommFault() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.FUEL_SENSOR_COMM_FAULT, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setFuelSensorCommFault(alarmStatusMap.get(Util.AlarmNames.FUEL_SENSOR_COMM_FAULT));
        }

        //New Alarms
        if (alarmStatusMap.get(Util.AlarmNames.BATTERY_FAULT) == null) {
            monitor.setBatteryFault(checkBatteryFault(model, monitor));
        } else {
            monitor.setBatteryFault(alarmStatusMap.get(Util.AlarmNames.BATTERY_FAULT));
        }

        if (alarmStatusMap.get(Util.AlarmNames.PV_MODULE_FAIL) == null) {
            monitor.setPvModuleFail(checkPvModuleFail(model, monitor));
        } else {
            monitor.setPvModuleFail(alarmStatusMap.get(Util.AlarmNames.PV_MODULE_FAIL));
        }

        if (alarmStatusMap.get(Util.AlarmNames.RECTIFIER_MODULE_FAIL) == null) {
            monitor.setRectifierModuleFail(
                    model.isdI8Spare() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.RECTIFIER_MODULE_FAIL, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setRectifierModuleFail(alarmStatusMap.get(Util.AlarmNames.RECTIFIER_MODULE_FAIL));
        }

        if (alarmStatusMap.get(Util.AlarmNames.DG_COMMON_FAULT) == null) {
            monitor.setDgCommonFault(
                    model.getdGCommonFault() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.DG_COMMON_FAULT, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDgCommonFault(alarmStatusMap.get(Util.AlarmNames.DG_COMMON_FAULT));
        }

        if (alarmStatusMap.get(Util.AlarmNames.SURGE_PROTECTION_DEVICE_FAIL) == null) {
            monitor.setSurgeProtectionDeviceFail(
                    model.isdI12Spare() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.SURGE_PROTECTION_DEVICE_FAIL, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setSurgeProtectionDeviceFail(alarmStatusMap.get(Util.AlarmNames.SURGE_PROTECTION_DEVICE_FAIL));
        }

        if (alarmStatusMap.get(Util.AlarmNames.LI_COMM_FAULT) == null) {
            monitor.setLiCommFault(checkLiCommFault(model, monitor));
        } else {
            monitor.setLiCommFault(alarmStatusMap.get(Util.AlarmNames.LI_COMM_FAULT));
        }

        if (alarmStatusMap.get(Util.AlarmNames.DC_AIRCON1_COMM_FAULT) == null) {
            monitor.setDcAirCon1CommFault(
                    model.isdCAirCon1CommFault() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.DC_AIRCON1_COMM_FAULT, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDcAirCon1CommFault(alarmStatusMap.get(Util.AlarmNames.DC_AIRCON1_COMM_FAULT));
        }

        if (alarmStatusMap.get(Util.AlarmNames.DC_AIRCON_2_COMM_FAULT) == null) {
            monitor.setDcAirCon2CommFault(
                    model.isdCAirCon2CommFault() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.DC_AIRCON_2_COMM_FAULT, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDcAirCon2CommFault(alarmStatusMap.get(Util.AlarmNames.DC_AIRCON_2_COMM_FAULT));
        }

        if (alarmStatusMap.get(Util.AlarmNames.DG_EMERGENCY_STOP) == null) {
            monitor.setDgEmergencyStop(
                    model.isdGEmergencyStop() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.DG_EMERGENCY_STOP, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDgEmergencyStop(alarmStatusMap.get(Util.AlarmNames.DG_EMERGENCY_STOP));
        }

        if (alarmStatusMap.get(Util.AlarmNames.DG_OVER_LOAD) == null) {
            monitor.setDgOverLoad(
                    model.isdGOverLoad() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.DG_OVER_LOAD, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDgOverLoad(alarmStatusMap.get(Util.AlarmNames.DG_OVER_LOAD));
        }

        if (alarmStatusMap.get(Util.AlarmNames.DG_ON_WITHOUT_LOAD) == null) {
            monitor.setDgOnWithoutLoud(
                    model.isdG12OnWithoutLoad() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.DG_ON_WITHOUT_LOAD, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDgOnWithoutLoud(alarmStatusMap.get(Util.AlarmNames.DG_ON_WITHOUT_LOAD));
        }

        if (alarmStatusMap.get(Util.AlarmNames.DG_FAIL_TO_STOP) == null) {
            monitor.setDgFailToStop(
                    model.isdGFailToStop() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.DG_FAIL_TO_STOP, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDgFailToStop(alarmStatusMap.get(Util.AlarmNames.DG_FAIL_TO_STOP));
        }

        if (alarmStatusMap.get(Util.AlarmNames.DG_LLOP) == null) {
            monitor.setDgLlop(
                    model.isLowLubeOilPressure() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.DG_LLOP, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDgLlop(alarmStatusMap.get(Util.AlarmNames.DG_LLOP));
        }

        if (alarmStatusMap.get(Util.AlarmNames.DG_HWT) == null) {
            monitor.setDgHwt(
                    model.isdGHwtHct() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.DG_HWT, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDgHwt(alarmStatusMap.get(Util.AlarmNames.DG_HWT));
        }

        if (alarmStatusMap.get(Util.AlarmNames.GPS_FIXED) == null) {
            monitor.setGpsFixed(
                    model.getGpsFix() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.GPS_FIXED, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setGpsFixed(alarmStatusMap.get(Util.AlarmNames.GPS_FIXED));
        }

        if (alarmStatusMap.get(Util.AlarmNames.FAN_FAIL) == null) {
            monitor.setFanFail(
                    model.isFanFailDI1() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.FAN_FAIL, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setFanFail(alarmStatusMap.get(Util.AlarmNames.FAN_FAIL));
        }

        if (alarmStatusMap.get(Util.AlarmNames.SYSTEM_IN_MANUAL_MODE) == null) {
            monitor.setSystemInManualMode(
                    model.isdI4Spare() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.SYSTEM_IN_MANUAL_MODE, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setSystemInManualMode(alarmStatusMap.get(Util.AlarmNames.SYSTEM_IN_MANUAL_MODE));
        }

        if (alarmStatusMap.get(Util.AlarmNames.TAMPER_SWITCH) == null) {
            monitor.setTamperSwitch(
                    model.isTamperSwitchforModem() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.TAMPER_SWITCH, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setTamperSwitch(alarmStatusMap.get(Util.AlarmNames.TAMPER_SWITCH));
        }

        if (alarmStatusMap.get(Util.AlarmNames.BLVD_TRIP) == null) {
            monitor.setBlvdTrip(
                    model.isBlvdTrip() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.BLVD_TRIP, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setBlvdTrip(alarmStatusMap.get(Util.AlarmNames.BLVD_TRIP));
        }

        if (alarmStatusMap.get(Util.AlarmNames.LLVD_TRIP) == null) {
            monitor.setLlvdTrip(
                    model.isLvdDisconnect() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.LLVD_TRIP, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setLlvdTrip(alarmStatusMap.get(Util.AlarmNames.LLVD_TRIP));
        }

        if (alarmStatusMap.get(Util.AlarmNames.DC_AIRCON_COMPRESSOR_FAN_FAULT) == null) {
            monitor.setDcAirConCompressorFanFault(checkDcAirConCompressorFanFault(model, monitor));
        } else {
            monitor.setDcAirConCompressorFanFault(alarmStatusMap.get(Util.AlarmNames.DC_AIRCON_COMPRESSOR_FAN_FAULT));
        }

        if (alarmStatusMap.get(Util.AlarmNames.SINGLE_RECTIFIER_FAIL) == null) {
            monitor.setSingleRectifierFault(
                    model.isSingleRectifierfail() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.SINGLE_RECTIFIER_FAIL, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setSingleRectifierFault(alarmStatusMap.get(Util.AlarmNames.SINGLE_RECTIFIER_FAIL));
        }

        if (alarmStatusMap.get(Util.AlarmNames.MULTIPLE_RECTIFIER_FAIL) == null) {
            monitor.setMultipleRectifierFault(
                    model.isMultipleRectifierFail() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.MULTIPLE_RECTIFIER_FAIL, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setMultipleRectifierFault(alarmStatusMap.get(Util.AlarmNames.MULTIPLE_RECTIFIER_FAIL));
        }

        if (alarmStatusMap.get(Util.AlarmNames.FUEL_ADDITION) == null) {
            monitor.setFuelAdditionAlarm(
                    model.getFuelAdditionAlarm() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.FUEL_ADDITION, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setFuelAdditionAlarm(alarmStatusMap.get(Util.AlarmNames.FUEL_ADDITION));
        }

        if (alarmStatusMap.get(Util.AlarmNames.FUEL_PILFERAGE) == null) {
            monitor.setFuelPilferageAlarm(
                    model.getFuelPilferageAlarm() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.FUEL_PILFERAGE, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setFuelPilferageAlarm(alarmStatusMap.get(Util.AlarmNames.FUEL_PILFERAGE));
        }

        if (alarmStatusMap.get(Util.AlarmNames.AC_SURGE_PROTECTION_DEVICE_FAIL) == null) {
            monitor.setACSurgeProtectionDeviceFail(
                    model.isaCSurgeProtectionDeviceFail() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.AC_SURGE_PROTECTION_DEVICE_FAIL, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setACSurgeProtectionDeviceFail(alarmStatusMap.get(Util.AlarmNames.AC_SURGE_PROTECTION_DEVICE_FAIL));
        }

        if (alarmStatusMap.get(Util.AlarmNames.DC_SURGE_PROTECTION_DEVICE_FAIL) == null) {
            monitor.setDCSurgeProtectionDeviceFail(
                    model.isdCSurgeProtectionDeviceFail() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.DC_SURGE_PROTECTION_DEVICE_FAIL, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setDCSurgeProtectionDeviceFail(alarmStatusMap.get(Util.AlarmNames.DC_SURGE_PROTECTION_DEVICE_FAIL));
        }

        if (alarmStatusMap.get(Util.AlarmNames.AIRCON_FAIL) == null) {
            monitor.setAirConFail(
                    model.getSonicDI16() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.AIRCON_FAIL, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setAirConFail(alarmStatusMap.get(Util.AlarmNames.AIRCON_FAIL));
        }

        if (alarmStatusMap.get(Util.AlarmNames.SYSTEM_CONTROLLER_DI5) == null) {
            monitor.setEbFail(
                    model.getSystemControllerDI5() == 1
                            ? Util.getAlarmCriticality(Util.AlarmNames.SYSTEM_CONTROLLER_DI5, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setEbFail(alarmStatusMap.get(Util.AlarmNames.SYSTEM_CONTROLLER_DI5));
        }

        if (alarmStatusMap.get(Util.AlarmNames.SOC_BATTERY_1) == null) {
            monitor.setSocBattery1(
                    (int) model.getSocBattery1() <= 28
                            ? Util.getAlarmCriticality(Util.AlarmNames.SOC_BATTERY_1, monitor.getAlarmConfigMap())
                            : Util.AlarmStatus.NORMAL_ALARM
            );
        } else {
            monitor.setSocBattery1(alarmStatusMap.get(Util.AlarmNames.SOC_BATTERY_1));
        }

/*
        monitor.setBatteryFault(checkBatteryFault(model, monitor));
        monitor.setPvModuleFail(checkPvModuleFail(model, monitor));

        monitor.setRectifierModuleFail(model.isdI8Spare() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.RECTIFIER_MODULE_FAIL, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setDgCommonFault(model.getdGCommonFault() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.DG_COMMON_FAULT, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setSurgeProtectionDeviceFail(model.isdI12Spare() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.SURGE_PROTECTION_DEVICE_FAIL, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setLiCommFault(checkLiCommFault(model, monitor));
        monitor.setDcAirCon1CommFault(model.isdCAirCon1CommFault() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.DC_AIRCON1_COMM_FAULT, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setDcAirCon2CommFault(model.isdCAirCon2CommFault() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.DC_AIRCON_2_COMM_FAULT, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setDgEmergencyStop(model.isdGEmergencyStop() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.DG_EMERGENCY_STOP, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setDgOverLoad(model.isdGOverLoad() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.DG_OVER_LOAD, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setDgOnWithoutLoud(model.isdG12OnWithoutLoad() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.DG_ON_WITHOUT_LOAD, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setDgFailToStop(model.isdGFailToStop() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.DG_FAIL_TO_STOP, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setDgLlop(model.isLowLubeOilPressure() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.DG_LLOP, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setDgHwt(model.isdGHwtHct() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.DG_HWT, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setGpsFixed(model.getGpsFix() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.GPS_FIXED, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setFanFail(model.isFanFailDI1() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.FAN_FAIL, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setSystemInManualMode(model.isdI4Spare() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.SYSTEM_IN_MANUAL_MODE, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setTamperSwitch(model.isTamperSwitchforModem() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.TAMPER_SWITCH, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setBlvdTrip(model.isBlvdTrip() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.BLVD_TRIP, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setLlvdTrip(model.isLvdDisconnect() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.LLVD_TRIP, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setDcAirConCompressorFanFault(checkDcAirConCompressorFanFault(model, monitor));
        monitor.setSingleRectifierFault(model.isSingleRectifierfail() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.SINGLE_RECTIFIER_FAIL, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setMultipleRectifierFault(model.isMultipleRectifierFail() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.MULTIPLE_RECTIFIER_FAIL, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setFuelAdditionAlarm(model.getFuelAdditionAlarm() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.FUEL_ADDITION, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setFuelPilferageAlarm(model.getFuelPilferageAlarm() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.FUEL_PILFERAGE, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);

        monitor.setTenantName(monitor.getClientId().endsWith("-GT") ? "Globe" : monitor.getClientId().endsWith("-SC") ? "Smart" : "N/A");

        monitor.setACSurgeProtectionDeviceFail(model.isaCSurgeProtectionDeviceFail() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.AC_SURGE_PROTECTION_DEVICE_FAIL, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setDCSurgeProtectionDeviceFail(model.isdCSurgeProtectionDeviceFail() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.DC_SURGE_PROTECTION_DEVICE_FAIL, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setAirConFail(model.getSonicDI16() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.AIRCON_FAIL, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setEbFail(model.getSystemControllerDI5() == 1 ? Util.getAlarmCriticality(Util.AlarmNames.SYSTEM_CONTROLLER_DI5, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
        monitor.setSocBattery1(((int) model.getSocBattery1()) <= 28 ? Util.getAlarmCriticality(Util.AlarmNames.SOC_BATTERY_1, monitor.getAlarmConfigMap()) : Util.AlarmStatus.NORMAL_ALARM);
*/

        monitor.setMainsOnBatteryDischarge(monitorScreenModel.getMainsOnBatteryDischarge());

        return monitor;
    }

    private String checkLiCommFault(PacketModel model, MonitorAlarmModel monitorModel) {
        String criticality = Util.AlarmStatus.NORMAL_ALARM;
        if (model.getLiCommFault1().equals("1") || model.getLiCommFault2().equals("1") || model.getLiCommFault3().equals("1")
                || model.getLiCommFault4().equals("1") || model.getLiCommFault5().equals("1") || model.getLiCommFault6().equals("1")
                || model.getLiCommFault7().equals("1") || model.getLiCommFault8().equals("1") || model.getLiCommFault9().equals("1")
                || model.getLiCommFault10().equals("1") || model.getLiCommFault11().equals("1") || model.getLiCommFault12().equals("1")
                || model.getLiCommFault13().equals("1") || model.getLiCommFault14().equals("1") || model.getLiCommFault15().equals("1")
                || model.getLiCommFault16().equals("1")) {

            criticality = Util.getAlarmCriticality(Util.AlarmNames.LI_COMM_FAULT, monitorModel.getAlarmConfigMap());
        }

        return criticality;
    }

    private String checkPvModuleFail(PacketModel model, MonitorAlarmModel monitorModel) {
        String criticality = Util.AlarmStatus.NORMAL_ALARM;
        if (model.isPv1ModuleStatus() == 1 || model.isPv2ModuleStatus() == 1 || model.isPv3ModuleStatus() == 1 || model.isPv4ModuleStatus() == 1
                || model.isPv5ModuleStatus() == 1 || model.isPv6ModuleStatus() == 1 || model.isPv7ModuleStatus() == 1 || model.isPv8ModuleStatus() == 1
                || model.isPv9ModuleStatus() == 1 || model.isPv10ModuleStatus() == 1 || model.isPv11ModuleStatus() == 1 || model.isPv12ModuleStatus() == 1) {
            criticality = Util.getAlarmCriticality(Util.AlarmNames.PV_MODULE_FAIL, monitorModel.getAlarmConfigMap());
        }

        return criticality;
    }

    private String checkBatteryFault(PacketModel model, MonitorAlarmModel monitorModel) {
        String criticality = Util.AlarmStatus.NORMAL_ALARM;
        if (model.getLiBattStatus1().equals("4") || model.getLiBattStatus2().equals("4") || model.getLiBattStatus3().equals("4")
                || model.getLiBattStatus4().equals("4") || model.getLiBattStatus5().equals("4") || model.getLiBattStatus6().equals("4")
                || model.getLiBattStatus7().equals("4") || model.getLiBattStatus8().equals("4") || model.getLiBattStatus9().equals("4")
                || model.getLiBattStatus10().equals("4") || model.getLiBattStatus11().equals("4") || model.getLiBattStatus12().equals("4")
                || model.getLiBattStatus13().equals("4") || model.getLiBattStatus14().equals("4") || model.getLiBattStatus15().equals("4")
                || model.getLiBattStatus16().equals("4")) {
            criticality = Util.getAlarmCriticality(Util.AlarmNames.BATTERY_FAULT, monitorModel.getAlarmConfigMap());
        }

        return criticality;
    }

    private String checkDcAirConCompressorFanFault(PacketModel model, MonitorAlarmModel monitorModel) {
        String criticality = Util.AlarmStatus.NORMAL_ALARM;
        if (model.isDcAircon1CompressorFaultFanfault() == 1 || model.isDcAircon2CompressorFaultFanfault() == 1) {
            criticality = Util.getAlarmCriticality(Util.AlarmNames.DC_AIRCON_COMPRESSOR_FAN_FAULT, monitorModel.getAlarmConfigMap());
        }
        return criticality;
    }

    public String getAlarmStatusFromConfig(String siteId, String alarmName) {
        Map<String, SiteWiseAlarmMaskingConfig> alarmConfigMap = siteWiseAlarmMaskingConfigService.getAlarmMaskingConfigMap(siteId);

        if (alarmConfigMap == null || !alarmConfigMap.containsKey(alarmName)) {
            return null; // Config not available, continue process
        }

        SiteWiseAlarmMaskingConfig alarmConfig = alarmConfigMap.get(alarmName);
        System.out.println(alarmConfig.getAlarmName() + ":" + alarmConfig.getAlarmEnabled());
        if ((alarmConfig.getAlarmEnabled())) {

            return null; // Alarm is enabled, continue process
        } else {
            return "N/A"; // Alarm is disabled
        }
    }

    public static List<String> getAllAlarms() {
        List<String> alarms = new ArrayList<>();
        Field[] fields = Util.AlarmNames.class.getDeclaredFields();

        for (Field field : fields) {
            try {
                alarms.add((String) field.get(null));
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        }
        return alarms;
    }
}
