package com.ieng.rms.services;

import com.ieng.rms.dto.ContractDTO;
import com.ieng.rms.dto.EscoConfigDTO;
import com.ieng.rms.feignclient.RMSCoreClient;
import com.ieng.rms.models.SiteDetailsDTO;
import com.ieng.rms.util.Constants;
import org.springframework.http.ResponseEntity;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class RmsCoreClientService {

    private final RMSCoreClient rmsCoreClient;

    public RmsCoreClientService(RMSCoreClient rmsCoreClient) {
        this.rmsCoreClient = rmsCoreClient;
    }

    // Updated the Code to handle SiteDetails not present
    public SiteDetailsDTO getSiteDetailsBySiteId(String siteId) {
        if (Constants.siteDetailsMap.containsKey(siteId)) {
            return getSiteDetailsBySiteIdFromMap(siteId);
        }
        SiteDetailsDTO siteDetailsDTO;
        siteDetailsDTO = getBody(rmsCoreClient.getSiteDetailsBySiteId(siteId), siteId);
        Constants.siteDetailsMap.put(siteId, siteDetailsDTO);
        return siteDetailsDTO;
    }

    public List<ContractDTO> getAllContracts() {
        return getBody(rmsCoreClient.getAllContracts());
    }

    private <T> T getBody(ResponseEntity<T> response, @Nullable String... strings) {
        validateParams(strings);

        if (response != null && response.getBody() != null) return response.getBody();
        return null;
    }

    private void validateParams(String... strings) {
        if (strings != null) {
            for (String str : strings) {
                if (str == null || str.isEmpty()) throw new RuntimeException("Param: " + str + " is Null");
            }
        }
    }

    public SiteDetailsDTO getSiteDetailsBySiteIdFromMap(String siteDetail) {
        return Constants.siteDetailsMap.get(siteDetail);
    }
    public EscoConfigDTO getEscoConfigByProtocolName(String protocolName) {
        return getBody(rmsCoreClient.getEscoConfigurationByProtocol(protocolName));
    }

    public List<SiteDetailsDTO> getAllSitesByContractId(Integer contractId) {
        return getBody(rmsCoreClient.getAllSiteInfoByContract(contractId), String.valueOf(contractId));
    }
}
