package com.ieng.rms.subscriber;

import com.ieng.rms.cache.overview.common.AlarmWithSeverityAndStateCacheObject;
import com.ieng.rms.dto.OldAlarmDetailReportDto;
import com.ieng.rms.entity.AlarmDetails;
import com.ieng.rms.entity.SiteWiseAlarmMaskingConfig;
import com.ieng.rms.models.PacketModel;
import com.ieng.rms.models.SiteDetailsDTO;
import com.ieng.rms.services.AlarmDetailConfigService;
import com.ieng.rms.services.RmsCoreClientService;
import com.ieng.rms.services.SiteWiseAlarmMaskingConfigService;
import com.ieng.rms.util.Constants;
import com.ieng.rms.util.RedisUtil;
import com.ieng.rms.util.Util;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.ieng.rms.util.RedisUtil.generateRedisKey;
import static com.ieng.rms.util.RedisUtil.getSiteOrClientId;

@Component
public class AlarmDetailCacheSubscriber {
    private static final Logger logger = LogManager.getLogger(AlarmDetailCacheSubscriber.class);

    @Autowired
    RmsCoreClientService rmsCoreClientService;
    private final SiteWiseAlarmMaskingConfigService siteWiseAlarmMaskingConfigService;

    @Autowired
    RedisUtil redisUtil;

    private final AlarmDetailConfigService alarmDetailConfigService;

    public AlarmDetailCacheSubscriber(SiteWiseAlarmMaskingConfigService siteWiseAlarmMaskingConfigService, AlarmDetailConfigService alarmDetailConfigService) {
        this.siteWiseAlarmMaskingConfigService = siteWiseAlarmMaskingConfigService;
        this.alarmDetailConfigService = alarmDetailConfigService;
    }

    @Async(value = "redisThreadExecutor")
    public void receive(PacketModel model, Boolean multiTenant, SiteDetailsDTO siteDetailsDTO) {
        try {

            Map<String, String> alarmConfigMap = alarmDetailConfigService.getAlarmConfigMapByContractId(siteDetailsDTO.getContractParentId());

            redisUtil.putToRedis(Constants.ALL_ALARMS, getSiteOrClientId(model, multiTenant), getAllAlarmsList(model, alarmConfigMap));
            redisUtil.putToRedis(Constants.ACTIVE_ALARMS, getSiteOrClientId(model, multiTenant), getActiveAlarmsList(model, alarmConfigMap));


        } catch (Exception ex) {
            logger.error("Exception occure while caching All and Active alarms........." + " Site ID: " + model.getSiteID() + " PID: " + model.getPacketId(), ex);
            ex.printStackTrace();
        }
    }

/*
    private List<OldAlarmDetailReportDto> getActiveAlarmsList(PacketModel model, Map<String, String> alarmConfigMap) {
        List<OldAlarmDetailReportDto> activeAlarms = new ArrayList<>();
        try {
            SiteDetailsDTO siteDetailsDTO = rmsCoreClientService.getSiteDetailsBySiteId(model.getSiteID());
            activeAlarms = Constants.alarmDetailMap.get(model.getSiteID()).entrySet().stream()
                    .map(p -> p.getValue())
                    .map(p -> new OldAlarmDetailReportDto(p, alarmConfigMap.get(Util.addUnderscoreAgainstSpace(p.getAlarmName())), siteDetailsDTO))
                    .collect(Collectors.toList());
        } catch (Exception e) {
            System.out.println(" ALARM MAP SIZE : " + alarmConfigMap.size());

            e.printStackTrace();
            logger.error("Exception while getting active alarm list :{}", e.getStackTrace());
        }
        return activeAlarms;
    }*/

/*	private List<AlarmWithSeverityAndStateCacheObject> getAllAlarmsList(PacketModel model, Map<String, String> alarmConfigMap) {
		List<AlarmWithSeverityAndStateCacheObject> alarmsList = new ArrayList<>();
		alarmConfigMap.entrySet().forEach(alarm->{
			String alarmCriticality = Constants.siteWiseAlarmsStatusMap.get(generateRedisKey(alarm.getKey(), model.getSiteID()));
			boolean alarmStatus = false;
			if(Util.AlarmStatus.NORMAL_ALARM.equals(alarmCriticality)){
				alarmStatus = false;
			}else{
				alarmStatus = true;
			}
			alarmsList.add(new AlarmWithSeverityAndStateCacheObject(Util.removeUnderscoreWithSpace(alarm.getKey()),alarm.getValue(),alarmStatus));
		});
		return alarmsList;
	}*/

    private List<OldAlarmDetailReportDto> getActiveAlarmsList(PacketModel model, Map<String, String> alarmConfigMap) {
        List<OldAlarmDetailReportDto> activeAlarms = new ArrayList<>();
        try {
            SiteDetailsDTO siteDetailsDTO = rmsCoreClientService.getSiteDetailsBySiteId(model.getSiteID());

            // Fetch site-wise alarm masking configurations from cache or DB
            Map<String, SiteWiseAlarmMaskingConfig> alarmConfigMapFromCache = siteWiseAlarmMaskingConfigService.getAlarmMaskingConfigMap(model.getSiteID());
            Map<String, Map<String, AlarmDetails>> alarmDetails = alarmDetailConfigService.loadAlarmDetailsFromRedisOrDB(model.getSiteID());

            // Create a mapping of alarmName (transformed) -> isEnabled
            Map<String, Boolean> alarmEnabledMap = alarmConfigMapFromCache != null ? alarmConfigMapFromCache.values().stream().collect(Collectors.toMap(config -> Util.addUnderscoreAgainstSpace(config.getAlarmName()), SiteWiseAlarmMaskingConfig::getAlarmEnabled)) : new HashMap<>();

            // Create a mapping of alarmName (transformed) -> alarmMaskingName for enabled alarms
            Map<String, String> alarmMaskingNameMap = alarmConfigMapFromCache != null ? alarmConfigMapFromCache.values().stream().filter(SiteWiseAlarmMaskingConfig::getAlarmEnabled).collect(Collectors.toMap(config -> Util.addUnderscoreAgainstSpace(config.getAlarmName()), SiteWiseAlarmMaskingConfig::getAlarmMaskingName)) : new HashMap<>();

            activeAlarms = alarmDetails.get(model.getSiteID()).values().stream().filter(alarm -> {
                // Use transformed alarm name as key
                String transformedKey = Util.addUnderscoreAgainstSpace(alarm.getAlarmName());
                Boolean isEnabled = alarmEnabledMap.get(transformedKey);
                return isEnabled == null || isEnabled; // Allow alarms that are enabled or not configured
            }).map(alarm -> {
                String transformedKey = Util.addUnderscoreAgainstSpace(alarm.getAlarmName());
                // Lookup the masking name using the transformed key
                String finalAlarmName = alarmMaskingNameMap.getOrDefault(transformedKey, alarm.getAlarmName());
                String alarmForCriticality = alarm.getAlarmName();
                alarm.setAlarmMaskingName(finalAlarmName);

                return new OldAlarmDetailReportDto(alarm, alarmConfigMap.get(Util.addUnderscoreAgainstSpace(alarmForCriticality)), siteDetailsDTO);
            }).collect(Collectors.toList());

        } catch (Exception e) {
            logger.info("ALARM MAP SIZE : " + (alarmConfigMap != null ? alarmConfigMap.size() : "NULL"));
            e.printStackTrace();
            logger.error("Exception while getting active alarm list: {}", e.getMessage());
        }
        return activeAlarms;
    }


    private List<AlarmWithSeverityAndStateCacheObject> getAllAlarmsList(PacketModel model, Map<String, String> alarmConfigMap) {
        List<AlarmWithSeverityAndStateCacheObject> alarmsList = new ArrayList<>();

        // Fetch site-wise alarm masking configurations from cache or DB
        Map<String, SiteWiseAlarmMaskingConfig> alarmConfigMapFromCache = siteWiseAlarmMaskingConfigService.getAlarmMaskingConfigMap(model.getSiteID());

        // Create a mapping of alarmName (transformed) -> isEnabled
        Map<String, Boolean> alarmEnabledMap = alarmConfigMapFromCache != null ? alarmConfigMapFromCache.values().stream().collect(Collectors.toMap(config -> Util.addUnderscoreAgainstSpace(config.getAlarmName()), SiteWiseAlarmMaskingConfig::getAlarmEnabled)) : new HashMap<>();

        // Create a mapping of alarmName (transformed) -> alarmMaskingName for enabled alarms
        Map<String, String> alarmMaskingNameMap = alarmConfigMapFromCache != null ? alarmConfigMapFromCache.values().stream().filter(SiteWiseAlarmMaskingConfig::getAlarmEnabled).collect(Collectors.toMap(config -> Util.addUnderscoreAgainstSpace(config.getAlarmName()), SiteWiseAlarmMaskingConfig::getAlarmMaskingName)) : new HashMap<>();

        if (alarmConfigMap != null) {
            alarmConfigMap.forEach((alarmKey, severity) -> {
                // Use transformed key for lookup
                String transformedKey = Util.addUnderscoreAgainstSpace(alarmKey);
                Boolean isEnabled = alarmEnabledMap.get(transformedKey);
                if (isEnabled != null && !isEnabled) {
                    return; // Skip this alarm if explicitly disabled
                }

                // Lookup the masking name
                String finalAlarmName = alarmMaskingNameMap.getOrDefault(transformedKey, alarmKey);

                // Determine alarm status based on site-wise alarm status map
                String alarmCriticality = Constants.siteWiseAlarmsStatusMap.get(generateRedisKey(alarmKey, model.getSiteID()));
                boolean alarmStatus = !Util.AlarmStatus.NORMAL_ALARM.equals(alarmCriticality);

                alarmsList.add(new AlarmWithSeverityAndStateCacheObject(Util.removeUnderscoreWithSpace(finalAlarmName), severity, alarmStatus));
            });
        } else {
            logger.warn("alarmConfigMap is null for siteID: {}", model.getSiteID());
        }

        return alarmsList;
    }


}
