package com.ieng.rms.subscriber;

import com.ieng.rms.models.MonitorAlarmModel;
import com.ieng.rms.models.PacketModel;
import com.ieng.rms.models.SiteDetailsDTO;
import com.ieng.rms.services.MonitorAlarmsService;
import com.ieng.rms.services.RmsCoreClientService;
import com.ieng.rms.websocket.AlarmMonitorWebsocketHandler;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.annotation.JmsListener;
import org.springframework.stereotype.Component;

@Component
public class AlarmsSubscriber {
    private static final Logger logger = LogManager.getLogger(AlarmsSubscriber.class);

    private final MonitorAlarmsService monitorAlarmsService;
    private final RmsCoreClientService rmsCoreClientService;
    private final AlarmDetailSubscriber alarmDetailSubscriber;
    private final AlarmMonitorWebsocketHandler alarmMonitorWebsocketHandler;

    @Autowired
    AlarmDetailCacheSubscriber alarmDetailCacheSubscriber;


    public AlarmsSubscriber(MonitorAlarmsService monitorAlarmsService, RmsCoreClientService rmsCoreClientService, AlarmDetailSubscriber alarmDetailSubscriber, AlarmMonitorWebsocketHandler alarmMonitorWebsocketHandler) {
        this.monitorAlarmsService = monitorAlarmsService;
        this.rmsCoreClientService = rmsCoreClientService;
        this.alarmDetailSubscriber = alarmDetailSubscriber;
        this.alarmMonitorWebsocketHandler = alarmMonitorWebsocketHandler;
    }

    @JmsListener(destination = "${jsa.activemq.monitor-alarms}", containerFactory = "jmsListenerContainerQueue")
    public void receive(PacketModel model) {
        SiteDetailsDTO site;
        String energyModel;

        try {
            if (model.getSiteID() != null) {
                site = validateSiteAndGet(model);
                energyModel = validateEnergyModelAndGet(site, model);
                alarmDetailSubscriber.receive(model);
                MonitorAlarmModel monitorModel = monitorAlarmsService.initMonitor(model, site, energyModel);
                alarmDetailCacheSubscriber.receive(model, site.getMultiTenant(), site);
                alarmMonitorWebsocketHandler.sendPacketToContract(monitorModel);
                logger.info("{} Alarm Stream to UI", model.getSiteID());
            }

        } catch (Exception e) {
            e.printStackTrace();
            logger.error("Exception occur while throwing Stream to UI ({}){}", model.getSiteID(), e);
        }

    }

    private SiteDetailsDTO validateSiteAndGet(PacketModel model) {
        return rmsCoreClientService.getSiteDetailsBySiteId(model.getSiteID().trim());
    }

    private String validateEnergyModelAndGet(SiteDetailsDTO site, PacketModel model) {
        String energyModel = null;
        if (site != null && site.getEnergyModelSimplifiedName() != null) {
            energyModel = site.getEnergyModelSimplifiedName();
        }
        if (energyModel == null) {
            logger.error("energy Model is missing for packet having site: {}", model.getSiteID());
        }
        return energyModel;
    }
}
