package com.ieng.rms.util;

import java.sql.Timestamp;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class DateUtil {
    public static LocalDateTime getLocalDateTime(Date date) {
        return LocalDateTime.ofInstant(Instant.ofEpochMilli(date.getTime()), ZoneId.systemDefault());
    }

    public static LocalDateTime getStringToLocalDateTime(String dateTimeStr) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        LocalDateTime dateTime = null;
        try {
            dateTime = LocalDateTime.parse(dateTimeStr, formatter);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dateTime;
    }
    public static LocalDateTime getStringToLocalDateTimeV1(String dateTimeStr) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss");

        LocalDateTime dateTime = null;
        try {
            dateTime = LocalDateTime.parse(dateTimeStr, formatter);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dateTime;
    }
    public static String convertDateToString(Date date) {
        LocalDateTime localDateTime = getLocalDateTime(date);
        // Formatting LocalDateTime to include seconds
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        return localDateTime.format(formatter);
    }

    public static long differenceInMilli(LocalDateTime startLocalDateTime, LocalDateTime endLocalDateTime) {
        return Duration.between(startLocalDateTime, endLocalDateTime).toMillis();
    }

    public static String millisecondsToTime(long milliSeconds) {
        long seconds = milliSeconds / 1000;
        long days = seconds / (3600 * 24);
        long hours = seconds / 3600;
        long min = (seconds / 60) % 60;
        long sec = seconds % 60;
        return String.format("%02d:%02d:%02d", hours, min, sec);
    }

    public static List<LocalDate> getAllDates(LocalDateTime startTime, LocalDateTime endTime) {
        List<LocalDate> allDates = DateUtil.allDatesBetween(startTime.toLocalDate(), endTime.toLocalDate());
        if (!allDates.contains(endTime.toLocalDate())) {
            allDates.add(endTime.toLocalDate());
        }
        return allDates;
    }

    public static List<LocalDate> allDatesBetween(LocalDate from, LocalDate to) {
        long numOfDays = ChronoUnit.DAYS.between(from, to);

        return Stream.iterate(from, date -> date.plusDays(1))
                .limit(numOfDays)
                .collect(Collectors.toList());
    }

    public static LocalDateTime atStartOfDay(LocalDate date) {
        return date.atStartOfDay().with(LocalTime.MIN);
    }

    public static Timestamp toTimeStamp(LocalDateTime localDateTime) {
        return Timestamp.valueOf(localDateTime);
    }

    public static LocalDateTime atEndOfDay(LocalDate date) {
        return date.atStartOfDay().with(LocalTime.MAX);
    }

    public static Timestamp atStartOfDay(String localDate) {
        return toTimeStamp(LocalDate.parse(localDate).atStartOfDay());
    }

    public static Timestamp atEndOfDay(String localDate) {
        return toTimeStamp(toEndOfDayTime(LocalDate.parse(localDate)));
    }

    public static LocalDateTime toEndOfDayTime(LocalDate localDate) {
        return localDate.atTime(LocalTime.MAX);
    }

    public static LocalDate getDateFromZone(String timezone) {
        return ZonedDateTime.now(ZoneId.of(timezone)).toLocalDate();
    }

    public static long getMilli(LocalDateTime dateTime) {
        return dateTime.atZone(ZoneId.systemDefault()).toInstant().toEpochMilli();
    }
}
