package com.ieng.rms.websocket;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.ieng.rms.dto.AlarmDetailReportDto;
import com.ieng.rms.entity.AlarmDetails;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.handler.TextWebSocketHandler;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Component
@Slf4j
public class AlarmWebSocketHandler extends TextWebSocketHandler {
    private final List<WebSocketSession> sessions = new CopyOnWriteArrayList<>();
    private final ExecutorService executorService = Executors.newFixedThreadPool(10);  // Use a thread pool of 10 threads

    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
        log.info("Connected");
        sessions.add(session);
        if (session.isOpen()) {
            log.info("open");
            session.sendMessage(new TextMessage("Connected"));
        } else {
            log.info("not open");
        }
    }

    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus status) {
        sessions.remove(session);
    }

    public void sendNewAlarm(AlarmDetails alarmDetailReportDto) {
        executorService.execute(() -> {
            try {
                String jsonData = new ObjectMapper().writeValueAsString(getAlarmDetailReportDto(alarmDetailReportDto));
                for (WebSocketSession session : sessions) {
                    session.sendMessage(new TextMessage(jsonData));
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        });
    }

    private AlarmDetailReportDto getAlarmDetailReportDto(AlarmDetails alarmDetails) {
        AlarmDetailReportDto alarmDetailReportDto = new AlarmDetailReportDto();
        alarmDetailReportDto.setSiteId(alarmDetails.getSiteId());
        alarmDetailReportDto.setAlarmName(alarmDetails.getAlarmName());
        alarmDetailReportDto.setAlarmStatus(alarmDetails.getAlarmStatus());
        alarmDetailReportDto.setAlarmCriticality(alarmDetails.getAlarmCriticality());
        alarmDetailReportDto.setFinalPacketId(alarmDetails.getFinalPacketId());
        alarmDetailReportDto.setElapsedTime(alarmDetails.getElapsedTime());
        alarmDetailReportDto.setFromTime(String.valueOf(alarmDetails.getFromTime()));
        alarmDetailReportDto.setInitialPacketId(alarmDetails.getInitialPacketId());
        alarmDetailReportDto.setToTime(String.valueOf(alarmDetails.getToTime()));
        alarmDetailReportDto.setAlarmName(alarmDetailReportDto.getSiteName() + "_" + alarmDetailReportDto.getAlarmName());
        alarmDetailReportDto.setAcknowledge(false);
        return alarmDetailReportDto;
    }
}

