package com.ieng.rms.websocket;

import com.ieng.rms.dto.ContractDTO;
import com.ieng.rms.services.RmsCoreClientService;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.socket.config.annotation.EnableWebSocket;
import org.springframework.web.socket.config.annotation.WebSocketConfigurer;
import org.springframework.web.socket.config.annotation.WebSocketHandlerRegistry;

import java.util.List;

import static com.ieng.rms.util.Constants.websocketEndPoint;
import static com.ieng.rms.util.Constants.websocketEndPointForAlarms;

@Configuration
@EnableWebSocket
public class WebSocketConfig implements WebSocketConfigurer {
    private static final Logger logger = LogManager.getLogger(WebSocketConfig.class);

    private final AlarmMonitorWebsocketHandler alarmMonitorWebsocketHandler;
    private final AlarmWebSocketHandler alarmWebSocketHandler;
    private final RmsCoreClientService rmsCoreClientService;

    public WebSocketConfig(AlarmMonitorWebsocketHandler alarmMonitorWebsocketHandler, AlarmWebSocketHandler alarmWebSocketHandler, RmsCoreClientService rmsCoreClientService) {
        this.alarmMonitorWebsocketHandler = alarmMonitorWebsocketHandler;
        this.alarmWebSocketHandler = alarmWebSocketHandler;
        this.rmsCoreClientService = rmsCoreClientService;
    }

    @Override
    public void registerWebSocketHandlers(WebSocketHandlerRegistry registry) {
        List<ContractDTO> allContracts = rmsCoreClientService.getAllContracts();
        allContracts.forEach(contract -> {
            String endpoint = websocketEndPoint + contract.getId();
            registry.addHandler(alarmMonitorWebsocketHandler, endpoint).setAllowedOrigins("*");
            logger.info("WebSocket endpoint registered: " + endpoint);
        });

        registry.addHandler(alarmWebSocketHandler, websocketEndPointForAlarms).setAllowedOrigins("*");

    }

}